﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_combat.lua
--
--    Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Actions module for combat rogues
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Rogue then return end -- need clcInfo_Rogue
local _, englishClass = UnitClass("player") -- need to be a rogue
if englishClass ~= clcInfo_Rogue.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Rogue = clcInfo_Rogue
local spells = clcInfo_Rogue.spells
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft
local GetCooldown = clcInfo_Rogue.GetCooldown
local IconNext = clcInfo_Rogue.IconNext
local IconCurrent = clcInfo_Rogue.IconCurrent
local BleedDebuffLeftOnTarget = clcInfo_Rogue.BleedDebuffLeftOnTarget
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
clcInfo_Rogue.COMBAT_MAX_NRG_SPECIALS = 50
local MAX_NRG_SPECIALS = clcInfo_Rogue.COMBAT_MAX_NRG_SPECIALS

-- Assign module variables
local specname = "combat"
local prio_arg = "combatprio"
local sample_prio = "tricks snd ks ar evisdeep rupture eviscerate rs ss"
local priority = {"none","snd","none","none","evisdeep","rupture",
                  "eviscerate","rs","ss"}
local class_module
local class_module_name

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Rogue.VERSION,
    priority = priority,
    ignore_rupture_no_bleed = true,
    rangePerSkill = false,
}
local tdb

-- actions --------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local AdrenalineRush = spells["Adrenaline Rush"].name
local BladeFlurry = spells["Blade Flurry"].name
local DeepInsight = spells["Deep Insight"].name
local ModerateInsight =  spells["Moderate Insight"].name
local RevealingStrike = spells["Revealing Strike"].name
local Rupture = spells["Rupture"].name
local ShallowInsight = spells["Shallow Insight"].name
local SliceAndDice = spells["Slice and Dice"].name
local FuryoftheDestroyer = spells["Fury of the Destroyer"].name

local AdrenalineRushId = spells["Adrenaline Rush"].id
local EviscerateId = spells["Eviscerate"].id
local KillingSpreeId = spells["Killing Spree"].id
local RevealingStrikeId = spells["Revealing Strike"].id
local RuptureId = spells["Rupture"].id
local SinisterStrikeId = spells["Sinister Strike"].id
local SliceAndDiceId = spells["Slice and Dice"].id
local TricksId = spells["Tricks of the Trade"].id

local actions  = {
    tricks = {
        id = TricksId,
        GetCD = function(cur_id)
            if cur_id ~= TricksId and (GetNumRaidMembers() > 0
                                       or GetNumPartyMembers() > 0) then
                return GetCooldown(TricksId)
            end
            return 100
        end,
        UpdateStatus = function(combo_points)
            return combo_points
        end,
        info = spells["Tricks of the Trade"].name,
    },
    snd = {
        id = SliceAndDiceId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= SliceAndDiceId 
               and combo_points > 0
               and GetTimeLeft("player", SliceAndDice, "HELPFUL|PLAYER") <= 1
            then
                return GetCooldown(SliceAndDiceId)
            end
            return 100
        end,
        UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
        end,
        info = spells["Slice and Dice"].name,
    },
    ks = {
        id = KillingSpreeId,
        GetCD = function(cur_id)
            if cur_id ~= KillingSpreeId
               and UnitPower("player") < MAX_NRG_SPECIALS
               and GetTimeLeft("player", SliceAndDice, "HELPFUL") > 4
               and GetTimeLeft("player", AdrenalineRush, "HELPFUL") == 0
               and GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") == 0
            then
                return GetCooldown(KillingSpreeId)
            end
            return 100
        end,
        UpdateStatus = function(combo_points)
            return combo_points
        end,
        info = spells["Killing Spree"].name,
    },
    ar = {
        id = AdrenalineRushId,
        GetCD = function(cur_id)
            if cur_id ~= AdrenalineRushId
               and UnitPower("player") < MAX_NRG_SPECIALS
            then
                return GetCooldown(AdrenalineRushId)
            end
            return 100
        end,
        UpdateStatus = function(combo_points)
            return combo_points, true -- ignore time increment since not on gcd
        end,
        info = spells["Adrenaline Rush"].name,
    },
    evisdeep = {
        id = EviscerateId,
        GetCD = function(_, combo_points)
            if combo_points == 5 
               and GetTimeLeft("player", DeepInsight, "HELPFUL") > 0
            then
                return GetCooldown(EviscerateId)
            end
            return 100
        end,
        UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
        end,
        info = format([[%s: %s]], spells["Eviscerate"].name,
                      spells["Deep Insight"].name),
    },
    rupture = {
        id = RuptureId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= RuptureId
               and combo_points == 5 
               and GetTimeLeft("target", Rupture, "HARMFUL|PLAYER") < 1
               and not (UnitAura("player", BladeFlurry, nil, "HELPFUL"))
               and (not tdb.ignore_rupture_no_bleed
                    or BleedDebuffLeftOnTarget() > 0)
            then
               return GetCooldown(RuptureId)
            end
            return 100
        end,
        UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
        end,
        info = spells["Rupture"].name,
    },
    eviscerate = {
        id = EviscerateId,
        GetCD = function(_, combo_points)
            if combo_points == 5 then
                return GetCooldown(EviscerateId)
            end
            return 100
        end,
        UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
        end,
        info = spells["Eviscerate"].name,
    },
    rs = {
        id = RevealingStrikeId,
        GetCD = function(_, combo_points)
            if combo_points == 4
              and GetTimeLeft("player", SliceAndDice, "HELPFUL") >= 3
              and GetTimeLeft("target", RevealingStrike, "HARMFUL|PLAYER") <= 1
            then
                return GetCooldown(RevealingStrikeId)
            end
            return 100
        end,
        UpdateStatus = function(combo_points)
            return combo_points + 1
        end,
        info = spells["Revealing Strike"].name,
    },
    ss = {
        id = SinisterStrikeId,
        GetCD = function(_, combo_points)
            if combo_points < 5 then
                return GetCooldown(SinisterStrikeId)
            end
            return 100
        end,
        UpdateStatus = function(combo_points)
            return combo_points + 1
        end,
        info = spells["Sinister Strike"].name,
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

--initialize module.
class_module, class_module_name = clcInfo_Rogue:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
      = clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions, tdb_default,
                                          specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Rogue:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadCombatModule()
    local _, talent_tree_name = GetTalentTabInfo(2)
    clcInfo_Rogue.LoadModule(class_module_name, talent_tree_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadCombatModule

-- giving queue as command line
local function CmdCombat(args)
    clcInfo_Rogue:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdCombat

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconCombatCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconCombatNext(...)
    return IconNext()
end
